#include <windows.h>
#include <windowsx.h>
#include "dialog.h"
#include "msgbox.h"
#include "external.h"
#include "resource.h"
#include "iprres.h"

// To align a pointer on dword boundary
#define PDWORD_ALIGN(p) (LPWORD)((((DWORD)(p)+3) >> 2) << 2)

char callback[MAX_PATH]="";

BOOL CALLBACK InputDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg){
	case WM_INITDIALOG:
		CreateInputWnd(14,50,210,35,hwnd,IDC_IP);
		SetFont(hwnd,IDC_IP,4);
		SetWindowText(GetDlgItem(hwnd, IDC_IP), "--");
		CentreWnd(hwnd);
		return 1;

    case WM_CLOSE:
		return 1;

    case WM_COMMAND:{
		switch (LOWORD(wParam)){
		case IDOK:
		case IDCANCEL:
			GetDlgItemText(hwnd, IDC_IP, callback, MAX_PATH);
			return EndDialog(hwnd, LOWORD(wParam));
		}
		}
	}
	return FALSE;
}

BOOL CALLBACK MessageDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg){
	case WM_INITDIALOG:
		CentreWnd(hwnd);
		return 1;

    case WM_COMMAND:{
		switch (LOWORD(wParam))
		case IDOK:
		case IDCANCEL:
			return EndDialog(hwnd, LOWORD(wParam));
		}
	}
	return FALSE;
}

LPWORD MakeDlg(LPDLGTEMPLATE pdlgt, DWORD style, DWORD exstyle, int num, short x, short y,
		short cx, short cy, short menu, short class, LPSTR szTitle, short pt, LPSTR facename)
{
	LPWORD lpw;
	LPWSTR lpwsz;

	//==============================================================
	// First, the pointer of allocated memory is passed as pdlgt,
	// the struct is filled in with appropriate values.
	//==============================================================
	// Do dialog
	pdlgt->style = style;
	pdlgt->dwExtendedStyle = exstyle;// extended style
	pdlgt->cdit = num; 				 // number of controls
	pdlgt->x    = x;				 // left
	pdlgt->y    = y;				 // top
	pdlgt->cx   = cx;				 // width
	pdlgt->cy   = cy;				 // height

	//==============================================================
	// Second, the pointer is taken past DLGTEMPLATE struct, and
	// the three variable-length arrays that specify the menu,
	// class, and title for the dialog box are filled in.
	//==============================================================

	// Increment the DWORD pointer past the DLGTEMPLATE struct
	lpw = (LPWORD) (pdlgt+1);

	*lpw++ = menu;   // menu
	*lpw++ = class;  // class

	// Set the title (could use MultiByteToWideChar() here)
	for (lpwsz = (LPWSTR)lpw;
        *lpwsz++ = (WCHAR)*szTitle++;
    );

    lpw = (LPWORD)lpwsz;

	// Set font point size
	*lpw++ = pt;

	// Set the face name (could use MultiByteToWideChar() here)
	for (lpwsz = (LPWSTR)lpw;
        *lpwsz++ = (WCHAR)*facename++;
    );

	// Return lastest addressed
    lpw = (LPWORD)lpwsz;

	return lpw;

}

LPWORD MakeDlgItem(LPWORD lpw, DWORD style, DWORD exstyle, short x, short y, short cx,
		short cy, WORD id, WORD arclass, WORD wclass, LPSTR szTitle, int ord, WORD cb)
{
	LPDLGITEMTEMPLATE pdlgIt;
	LPWSTR lpwsz;

	//=====================================================
	//
	// The pointer is passed after creating the DLG itself.
	//
	// The pointer is then aligned on a DWORD boundary, the
	// DLGITEMTEMPLATE struct is filled in and then the class
	// and title arrays are filled in.
	//=====================================================

	lpw = PDWORD_ALIGN(lpw); // align DLGITEMTEMPLATE on DWORD boundary
	pdlgIt = (LPDLGITEMTEMPLATE) lpw;
	pdlgIt->style = style;
	pdlgIt->dwExtendedStyle = exstyle;	// extended style
	pdlgIt->x  = x;
	pdlgIt->y  = y;
	pdlgIt->cx = cx;
	pdlgIt->cy = cy;
	pdlgIt->id = id;  					// identifier

	// Increment the pointer past the DLGITEMTEMPLATE struct
	lpw = (LPWORD) (pdlgIt + 1);

	*lpw++ = arclass;	// arr
	*lpw++ = wclass;	// class

	if (ord){
		*lpw++ = 0xFFFF;	// title
		*lpw++ = ord;		// ordinal value of icon
	}else{

		if (szTitle != NULL){
			// Set button text (could use MultiByteToWideChar() here)
			for (lpwsz = (LPWSTR)lpw;
		        *lpwsz++ = (WCHAR)*szTitle++;
		    );
		    lpw = (LPWORD)lpwsz;
		}
	}

	*lpw++ = 0x0000;       // no creation data

	return lpw;

}

LRESULT InputBox(HINSTANCE hInst, HWND hwndOwner, char *szTitle, char *szMess)
{
	LPDLGTEMPLATE pdlgt;
	LPWORD lpw;
	LRESULT ret;

	//					font name      but     but
	char *pszStr[] = {"MS Sans Serif", "Submit"};

	// Allocate enought memory for DLGTEMPLATE + 4 * DLGITEMTEMPLATE + Unicode strings etc
	pdlgt = (LPDLGTEMPLATE)malloc(1024);

	if (!pdlgt)
		return -1;

	// Define the Dialogbox.
	DWORD style = DS_MODALFRAME | DS_SETFONT | WS_POPUP | WS_CAPTION | WS_SYSMENU | WS_VISIBLE;
	lpw = MakeDlg(pdlgt, style, 0, 4, 0, 0, 178, 96, 0, 0, szTitle, 8, pszStr[0]);

	// Define an OK button.
	style = WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON;
	lpw = MakeDlgItem(lpw, style, 0, 64, 75, 50, 14, IDOK, 0xFFFF, BUTTON, pszStr[1], 0, 0);

	// Define a static with icon.
	style = WS_CHILD | WS_VISIBLE | SS_ICON;
	lpw = MakeDlgItem(lpw, style, 0, 4, 4, 50, 14, -1, 0xFFFF, STATIC, NULL, IDAPPLICON, 0);

	// Define a static text control.
	style = WS_CHILD | WS_VISIBLE | SS_LEFT;
	lpw = MakeDlgItem(lpw, style, 0, 27, 7, 144, 60, ID_TEXT, 0xFFFF, STATIC, szMess, 0, 0);

	// Define a static control.
	style = WS_CHILD | WS_VISIBLE | SS_GRAYRECT;
	lpw = MakeDlgItem(lpw, style, 0, 7, 62, 164, 1, -1, 0xFFFF, STATIC, NULL, 0, 0);

	ret = DialogBoxIndirect(hInst, (LPDLGTEMPLATE)pdlgt, hwndOwner,	(DLGPROC)InputDialogProc);

	free(pdlgt);

	return ret;
}

LRESULT MsgBox(HINSTANCE hInst, HWND hwndOwner, char *szTitle, char *szMess)
{
	LPDLGTEMPLATE pdlgt;
	LPWORD lpw;
	LRESULT ret;

	//					font name      but     but
	char *pszStr[] = {"MS Sans Serif", "OK"};

	// Allocate enought memory for DLGTEMPLATE + 4 * DLGITEMTEMPLATE + Unicode strings etc
	pdlgt = (LPDLGTEMPLATE)malloc(1024);

	if (!pdlgt)
		return -1;

	// Define the Dialogbox.
	DWORD style = DS_MODALFRAME | DS_SETFONT | WS_POPUP | WS_CAPTION | WS_SYSMENU | WS_VISIBLE;
	lpw = MakeDlg(pdlgt, style, 0, 4, 0, 0, 178, 96, 0, 0, szTitle, 8, pszStr[0]);

	// Define an OK button.
	style = WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON;
	lpw = MakeDlgItem(lpw, style, 0, 64, 75, 50, 14, IDOK, 0xFFFF, BUTTON, pszStr[1], 0, 0);

	// Define a static with icon.
	style = WS_CHILD | WS_VISIBLE | SS_ICON;
	lpw = MakeDlgItem(lpw, style, 0, 4, 4, 50, 14, -1, 0xFFFF, STATIC, NULL, IDAPPLICON, 0);

	// Define a static text control.
	style = WS_CHILD | WS_VISIBLE | SS_LEFT;
	lpw = MakeDlgItem(lpw, style, 0, 27, 7, 144, 60, ID_TEXT, 0xFFFF, STATIC, szMess, 0, 0);

	// Define a static control.
	style = WS_CHILD | WS_VISIBLE | SS_GRAYRECT;
	lpw = MakeDlgItem(lpw, style, 0, 7, 62, 164, 1, -1, 0xFFFF, STATIC, NULL, 0, 0);

	ret = DialogBoxIndirect(hInst, (LPDLGTEMPLATE)pdlgt, hwndOwner,	(DLGPROC)MessageDialogProc);

	free(pdlgt);

	return ret;
}

